package ch.unige.solidify.controller;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.exception.SolidifyHttpErrorException;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.CompositeResource;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.SubResourceWithRelationContainer;
import ch.unige.solidify.service.CompositeResourceService;

public abstract class Relation3TiersController<T extends Resource<T> & SubResourceWithRelationContainer, V extends Resource<V>, W extends Resource<W>, Z extends CompositeResource<Z, ?>>
        extends RelationController<T, V, Z> {

  @Autowired
  protected CompositeResourceService<W> grandChildItemService;

  @PostMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<?> create(@PathVariable String parentid, @PathVariable String id,
          @RequestBody String[] ids) {
    final T parentItem = this.itemService.findOne(parentid);
    final V childItem = this.childItemService.findOne(id);
    final List<W> list = new ArrayList<>();
    for (final String i : ids) {
      final W grandChildItem = this.grandChildItemService.findOne(i);
      parentItem.addItem(childItem, grandChildItem);
      list.add(grandChildItem);
    }
    this.itemService.save(parentItem);
    return new ResponseEntity<>(list, HttpStatus.CREATED);
  }

  @PostMapping
  public HttpEntity<?> create(@PathVariable String parentid, @RequestBody String[] ids) {
    final T parentItem = this.itemService.findOne(parentid);
    final W grandChildItem = this.grandChildItemService.findOne(parentItem.getDefault());
    final List<W> list = new ArrayList<>();
    for (final String i : ids) {
      final V childItem = this.childItemService.findOne(i);
      parentItem.addItem(childItem, grandChildItem);
      list.add(grandChildItem);
    }
    this.itemService.save(parentItem);
    return new ResponseEntity<>(list, HttpStatus.CREATED);
  }

  @DeleteMapping(value = SolidifyConstants.URL_ID)
  public HttpStatus delete(@PathVariable String parentid, @PathVariable String id,
          @RequestBody(required = false) String[] ids) {
    final T parentItem = this.itemService.findOne(parentid);
    final V childItem = this.childItemService.findOne(id);
    final List<W> gdChildlist = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid, id);
    if (gdChildlist.isEmpty()) {
      throw new SolidifyHttpErrorException(HttpStatus.GONE, "Parent item " + parentid + " gone");
    }
    for (final W gdChildItem : gdChildlist) {
      if (this.isInList(ids, gdChildItem.getResId())) {
        parentItem.removeItem(childItem, gdChildItem);
      }
    }
    this.itemService.save(parentItem);
    return HttpStatus.OK;
  }

  @DeleteMapping
  public HttpStatus delete(@PathVariable String parentid,
          @RequestBody(required = false) String[] ids) {
    final T parentItem = this.itemService.findOne(parentid);
    final List<V> childList = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid);
    if (childList.isEmpty()) {
      throw new SolidifyHttpErrorException(HttpStatus.GONE, "Parent item " + parentid + " gone");
    }
    for (final V childItem : childList) {
      if (this.isInList(ids, childItem.getResId())) {
        final List<W> gdChildlist = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
                this.getMainResourceClass(), parentid, childItem.getResId());
        for (final W gdChildItem : gdChildlist) {
          parentItem.removeItem(childItem, gdChildItem);
        }
      }
    }

    this.itemService.save(parentItem);
    return HttpStatus.OK;
  }

  @GetMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<?> get(@PathVariable String parentid, @PathVariable String id) {
    final T parentItem = this.itemService.findOne(parentid);
    final V childItem = this.childItemService.findOne(id);
    final List<W> gdChildlist = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid, id);
    if (gdChildlist.isEmpty()) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    final Object newChild = parentItem.getDetails(childItem, gdChildlist);
    if (newChild == null) {
      return new ResponseEntity<>(HttpStatus.NOT_FOUND);
    }
    this.addLinks(parentid, (V) newChild);
    return new ResponseEntity<>(newChild, HttpStatus.OK);
  }

  @Override
  @SuppressWarnings("unchecked")
  @GetMapping
  public HttpEntity<Collection<?>> list(@PathVariable String parentid, Pageable pageable) {
    final T mainResourceItem = this.itemService.findOne(parentid);
    final Page<V> childList = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
            this.getMainResourceClass(), parentid, pageable);
    final List<Object> newList = new ArrayList<>();
    for (final V childItem : childList) {
      final List<W> gdChildlist = this.relationRepository.findByCompositeId(this.getCompositeResourceClass(),
              this.getMainResourceClass(), parentid, childItem.getResId());
      final Object newChild = mainResourceItem.getDetails(childItem, gdChildlist);
      this.addLinks(parentid, (V) newChild);
      newList.add(newChild);
    }

    final Collection<?> collection = this.processPageAndLinks(newList, childList, parentid, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @PatchMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<Z> update(@PathVariable String parentid, @PathVariable String id, @RequestBody Z z) {
    return new ResponseEntity<>(HttpStatus.NOT_IMPLEMENTED);
  }
}
