package ch.unige.solidify.controller;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.linkTo;
import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.methodOn;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.hateoas.server.mvc.WebMvcLinkBuilder;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import ch.unige.solidify.SolidifyConstants;
import ch.unige.solidify.rest.ActionName;
import ch.unige.solidify.rest.Collection;
import ch.unige.solidify.rest.Resource;
import ch.unige.solidify.rest.Tool;
import ch.unige.solidify.service.CompositeResourceService;
import ch.unige.solidify.util.SearchCriteria;
import ch.unige.solidify.util.StringParserTool;

public abstract class ResourceReadOnlyController<T extends Resource<T>> extends SolidifyController
        implements ControllerWithHateoasHome {

  @Autowired
  protected CompositeResourceService<T> itemService;

  @PostMapping(value = SolidifyConstants.URL_SEP + ActionName.SEARCH)
  public HttpEntity<Collection<T>> advancedSearch(@RequestBody T search,
          @RequestParam(value = "match", required = false) String matchtype, Pageable pageable) {
    final Page<T> listItem = this.itemService.findBySearchCriteria(search, matchtype,
            search.getSearchCriterias(), pageable);
    this.setRessourceLinks(listItem);
    final Collection<T> collection = this.setCollectionLinks(listItem, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @GetMapping(value = SolidifyConstants.URL_SEP + ActionName.SEARCH)
  @ResponseBody
  public HttpEntity<Collection<T>> advancedSearch(@ModelAttribute T resource,
          @RequestParam(value = "search") String search,
          @RequestParam(value = "match", required = false) String matchtype, Pageable pageable) {
    final List<SearchCriteria> criterias = StringParserTool.parseSearchString(search);
    final Page<T> listItem = this.itemService.findBySearchCriteria(resource, matchtype, criterias, pageable);
    final WebMvcLinkBuilder linkBuilder = linkTo(methodOn(this.getClass()).advancedSearch(null, search, matchtype, pageable));
    this.setRessourceLinks(listItem);
    final Collection<T> collection = this.setCollectionLinksForMethod(listItem, pageable, linkBuilder);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  @GetMapping(value = SolidifyConstants.URL_ID)
  public HttpEntity<T> get(@PathVariable String id) {
    final T t = this.itemService.findOne(id);
    this.addLinks(t);
    return new ResponseEntity<>(t, HttpStatus.OK);
  }

  @GetMapping
  public HttpEntity<Collection<T>> list(@ModelAttribute T search, Pageable pageable) {
    final Specification<T> spec = search.getSpecification();
    final Page<T> listItem = this.itemService.findAll(spec, pageable);
    this.setRessourceLinks(listItem);
    final Collection<T> collection = this.setCollectionLinks(listItem, pageable);
    return new ResponseEntity<>(collection, HttpStatus.OK);
  }

  protected void addLinks(T t) {
    t.removeLinks();
    t.addLinks(linkTo(this.getClass()), true, true);
  }

  protected <W extends RepresentationModel<W>> void addOthersLinks(W w) {
    // Do nothing
    // Could be overridden by subclass
  }

  protected Collection<T> setCollectionLinks(Page<T> listItem, Pageable pageable) {

    final Collection<T> collection = new Collection<>(listItem, pageable);
    collection.add(linkTo(this.getClass()).withSelfRel());
    collection.add(Tool.parentLink((linkTo(this.getClass())).toUriComponentsBuilder())
            .withRel(ActionName.MODULE));

    this.addSortLinks(linkTo(this.getClass()), collection);
    this.addPageLinks(linkTo(this.getClass()), collection, pageable);
    this.addOthersLinks(collection);
    return collection;
  }

  protected Collection<T> setCollectionLinksForMethod(Page<T> listItem, Pageable pageable, WebMvcLinkBuilder linkBuilder) {
    final Collection<T> collection = new Collection<>(listItem, pageable);
    collection.add(linkBuilder.withSelfRel());
    collection.add(Tool.parentLink((linkTo(this.getClass())).toUriComponentsBuilder())
            .withRel(ActionName.MODULE));
    this.addSortLinks(linkBuilder, collection);
    this.addPageLinks(linkBuilder, collection, pageable);
    this.addOthersLinks(collection);
    return collection;
  }

  protected void setRessourceLinks(Page<T> listItem) {

    if (listItem == null || listItem.getSize() == 0) {
      return;
    }

    for (final T t : listItem) {
      this.addLinks(t);
    }
  }

}
