package ch.unige.solidify.service;

import java.util.Locale;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.MessageSource;
import org.springframework.context.NoSuchMessageException;
import org.springframework.context.support.MessageSourceAccessor;
import org.springframework.stereotype.Service;

@Service
public class MessageService {
  private static final Logger log = LoggerFactory.getLogger(MessageService.class);

  private final MessageSourceAccessor accessor;

  @Autowired
  public MessageService(MessageSource messageSource) {
    this.accessor = new MessageSourceAccessor(messageSource, Locale.ENGLISH);
  }

  public String get(String code) {
    try {
      return this.accessor.getMessage(code);
    } catch (final NoSuchMessageException e) {
      return this.getErrorMessage(code, e);
    }
  }

  public String get(String code, Object[] args) {
    try {
      return this.accessor.getMessage(code, args);
    } catch (final NoSuchMessageException e) {
      return this.getErrorMessage(code, e);
    }
  }

  private String getErrorMessage(String code, NoSuchMessageException e) {
    log.error("Cannot find message", e);
    try {
      return this.accessor.getMessage("message.not.found", new Object[] { code });
    } catch (final NoSuchMessageException noSuchMessageException) {
      return "Message not found : " + code;
    }
  }
}
