package ch.unige.solidify.test.controller;

import static org.junit.Assert.assertEquals;

import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.test.context.junit4.SpringRunner;

import ch.unige.solidify.service.AuthenticationStorageProxy;
import ch.unige.solidify.service.MessageService;
import ch.unige.solidify.test.controller.service.ApplicationRestClientService;
import ch.unige.solidify.util.RestClientTool;

@RunWith(SpringRunner.class)
public class SolidifyRestClientServiceTest {

  /*
   * Mock class allowing to call completeUrlWithPagination with public access
   */
  private static class ApplicationRestClientServiceMock extends ApplicationRestClientService {
    public ApplicationRestClientServiceMock(MessageService messageService, RestClientTool restClientTool,
            AuthenticationStorageProxy authenticationStorageProxy) {
      super(messageService, restClientTool, authenticationStorageProxy);
    }

    @Override
    public String completeUrlWithPagination(String url, Pageable pageable) {
      return super.completeUrlWithPagination(url, pageable);
    }
  }

  private final String baseUrl = "http://localhost:8080/test";

  private ApplicationRestClientServiceMock restService;

  @Test
  public void completeBaseUrlWithNullPagination() {
    assertEquals(this.baseUrl, this.restService.completeUrlWithPagination(this.baseUrl, null));
  }

  @Test
  public void completeBaseUrlWithNullPaginationAndPageParameter() {
    final String url = this.baseUrl + "?page=45";
    assertEquals(url, this.restService.completeUrlWithPagination(url, null));
  }

  @Test
  public void completeUlrWithEncodedCharacter() {
    final String urlWithSpecialChars = this.baseUrl + "?a=%20";
    assertEquals(urlWithSpecialChars, this.restService.completeUrlWithPagination(urlWithSpecialChars, null));
  }

  @Test
  public void completeUlrWithExistingPaginationParameters() {
    final String url = this.baseUrl + "?page=0&size=10";
    final Pageable pageable = PageRequest.of(2, 5);
    final String pageableString = "?page=2&size=5";
    assertEquals(this.baseUrl + pageableString, this.restService.completeUrlWithPagination(url, pageable));
  }

  @Test
  public void completeUlrWithPlusCharacter() {
    final String urlWithPlusChar = this.baseUrl + "?a=toto+tot";
    assertEquals(urlWithPlusChar, this.restService.completeUrlWithPagination(urlWithPlusChar, null));
  }

  @Test
  public void completeUrlTest() {
    final String expectedResult = this.baseUrl + "?page=0&size=10";
    final Pageable pageable = PageRequest.of(0, 10);
    assertEquals(expectedResult, this.restService.completeUrlWithPagination(this.baseUrl, pageable));
  }

  @Test
  public void completeUrlWithParametersTest() {
    final String url = this.baseUrl + "?search=toto";
    final String expectedResult = url + "&page=0&size=10";
    final Pageable pageable = PageRequest.of(0, 10);
    assertEquals(expectedResult, this.restService.completeUrlWithPagination(url, pageable));
  }

  @Before
  public void setup() {
    this.restService = new ApplicationRestClientServiceMock(null, null, null);
  }

}
