package ch.unige.solidify.rest;

import java.time.OffsetDateTime;
import java.time.ZoneOffset;
import java.util.Objects;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.EntityListeners;
import javax.persistence.MappedSuperclass;
import javax.validation.Valid;

import org.springframework.data.jpa.domain.Specification;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.listener.CacheListener;
import ch.unige.solidify.message.CacheMessage;
import ch.unige.solidify.model.ChangeInfo;

@MappedSuperclass
@EntityListeners({ CacheListener.class })
public abstract class CompositeResource<T, K> extends ResourceBase {

  @AttributeOverride(name = "when", column = @Column(name = "creationWhen", length = 3))
  @AttributeOverride(name = "who", column = @Column(name = "creationWho"))
  @Embedded
  @Valid
  protected ChangeInfo creation = new ChangeInfo();

  @AttributeOverride(name = "when", column = @Column(name = "lastUpdateWhen", length = 3))
  @AttributeOverride(name = "who", column = @Column(name = "lastUpdateWho"))
  @Embedded
  @Valid
  protected ChangeInfo lastUpdate = new ChangeInfo();

  @Override
  public boolean equals(final Object o) {
    if (this == o) {
      return true;
    }
    if (!(o instanceof CompositeResource)) {
      return false;
    }
    if (!super.equals(o)) {
      return false;
    }
    final CompositeResource<T, K> that = (CompositeResource<T, K>) o;
    return Objects.equals(this.getCompositeResId(), that.getCompositeResId()) &&
            Objects.equals(this.getCreation(), that.getCreation()) &&
            Objects.equals(this.getLastUpdate(), that.getLastUpdate());
  }

  @JsonIgnore
  public CacheMessage getCacheMessage() {
    return null;
  }

  // *************************************************************************************

  public abstract K getCompositeResId();

  @Override
  @JsonIgnore
  public String getCreatedBy() {
    return this.getCreation().getWho();
  }

  @Override
  public ChangeInfo getCreation() {
    return this.creation;
  }

  @Override
  @JsonIgnore
  public OffsetDateTime getCreationTime() {
    return this.getCreation().getWhen();
  }

  @Override
  public ChangeInfo getLastUpdate() {
    return this.lastUpdate;
  }

  @JsonIgnore
  public abstract Specification<T> getSpecification();

  @Override
  @JsonIgnore
  public String getUpdatedBy() {
    return this.getLastUpdate().getWho();
  }

  @Override
  @JsonIgnore
  public OffsetDateTime getUpdateTime() {
    return this.getLastUpdate().getWhen();
  }

  @Override
  public int hashCode() {
    return Objects.hash(super.hashCode(), this.getCompositeResId(), this.getCreation(), this.getLastUpdate());
  }

  public abstract void setCompositeResId(K compositeResId);

  @Override
  public void setCreatedBy(String createdBy) {
    this.getCreation().setWho(createdBy);
    this.updateTime();
  }

  @Override
  public void setUpdatedBy(String updatedBy) {
    this.getLastUpdate().setWho(updatedBy);
    this.updateTime();
  }

  @Override
  public void updateTime() {
    this.getLastUpdate().setWhen(OffsetDateTime.now(ZoneOffset.UTC));
  }
}
