package ch.unige.solidify.rest;

import java.time.OffsetDateTime;
import java.time.ZoneOffset;

import javax.persistence.AttributeOverride;
import javax.persistence.Column;
import javax.persistence.Embedded;
import javax.persistence.EntityListeners;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;
import javax.validation.Valid;

import com.fasterxml.jackson.annotation.JsonIgnore;

import ch.unige.solidify.listener.CacheListener;
import ch.unige.solidify.model.ChangeInfo;
import ch.unige.solidify.util.StringTool;

@MappedSuperclass
@EntityListeners({ CacheListener.class })
public abstract class ResourceNormalized<T> extends Resource<T> {
  @AttributeOverride(name = "when", column = @Column(name = "creationWhen", length = 3))
  @AttributeOverride(name = "who", column = @Column(name = "creationWho"))
  @Embedded
  @Valid
  protected ChangeInfo creation = new ChangeInfo();

  @AttributeOverride(name = "when", column = @Column(name = "lastUpdateWhen", length = 3))
  @AttributeOverride(name = "who", column = @Column(name = "lastUpdateWho"))
  @Embedded
  @Valid
  protected ChangeInfo lastUpdate = new ChangeInfo();

  @Id
  private String resId;

  protected ResourceNormalized() {
    super();
    this.resId = StringTool.generateResId();
  }

  // *************************************************************************************

  @Override
  @JsonIgnore
  public String getCreatedBy() {
    return this.getCreation().getWho();
  }

  @Override
  public ChangeInfo getCreation() {
    return this.creation;
  }

  @Override
  @JsonIgnore
  public OffsetDateTime getCreationTime() {
    return this.getCreation().getWhen();
  }

  @Override
  public ChangeInfo getLastUpdate() {
    return this.lastUpdate;
  }

  @Override
  public String getResId() {
    return this.resId;
  }

  @Override
  @JsonIgnore
  public String getUpdatedBy() {
    return this.getLastUpdate().getWho();
  }

  @Override
  @JsonIgnore
  public OffsetDateTime getUpdateTime() {
    return this.getLastUpdate().getWhen();
  }

  @Override
  public void setCreatedBy(String createdBy) {
    this.getCreation().setWho(createdBy);
    this.updateTime();
  }

  @Override
  public void setResId(String resId) {
    this.resId = resId;
  }

  @Override
  public void setUpdatedBy(String updatedBy) {
    this.getLastUpdate().setWho(updatedBy);
    this.updateTime();
  }

  @Override
  public void updateTime() {
    this.getLastUpdate().setWhen(OffsetDateTime.now(ZoneOffset.UTC));
  }

}
